

#ifndef CEF_TESTS_CEFSIMPLE_SIMPLE_HANDLER_H_
#define CEF_TESTS_CEFSIMPLE_SIMPLE_HANDLER_H_

#include "include/cef_client.h"
#include <flutter/texture_registrar.h>
#include <list>
typedef struct
{
  int width;
  int height;
  double devicePixelRatio;
} Size;

enum class WebviewClientLoadState
{
  None,
  Start,
  Loading,
  Completed
};

struct WebviewClientHistoryChanged
{
  BOOL can_go_back;
  BOOL can_go_forward;
};

// enum class WebviewClientPermissionState
// {
//   Default,
//   Allow,
//   Deny
// };

enum class WebviewClientWindowOpenPolicy
{
  Allow,
  Deny,
 InSameWindow
};

enum class WebviewClientPointerButton
{
  None,
  Left,
  Right,
  Middle
};
enum class WebviewClientKeyEventType
{
  None,
  Down,
  Up,
  Char,
};
class WebviewClient : public CefClient,
                      public CefDisplayHandler,
                      public CefLifeSpanHandler,
                      public CefLoadHandler,
                      public CefRenderHandler,
                      public CefFocusHandler
{
public:
  typedef std::function<void()> FrameAvailableCallback;

  typedef std::function<void(const HCURSOR)> CursorChangedCallback;
  typedef std::function<void(WebviewClientLoadState)> LoadStateChangedCallback;
  typedef std::function<void(WebviewClientHistoryChanged)> HistoryChangedCallback;
  typedef std::function<void(const std::string &)> UrlChangedCallback;
  typedef std::function<void(const std::string &)> JSHandleCallback;

   WebviewClient(bool use_views, int width, int height, double devicePixelRatio);
  ~WebviewClient();

  void SetOnFrameAvailable(FrameAvailableCallback callback)
  {
    frame_available_ = std::move(callback);
  }
  // CefClient methods:
  virtual CefRefPtr<CefDisplayHandler> GetDisplayHandler() override
  {
    return this;
  }
  virtual CefRefPtr<CefLifeSpanHandler> GetLifeSpanHandler() override
  {
    return this;
  }
  virtual CefRefPtr<CefLoadHandler> GetLoadHandler() override
  {
    return this;
  }
  virtual CefRefPtr<CefRenderHandler> GetRenderHandler() override
  {
    return this;
  }

  virtual CefRefPtr<CefFocusHandler> GetFocusHandler() override
  {
    return this;
  }
  virtual bool OnProcessMessageReceived(CefRefPtr<CefBrowser> browser,
                                        CefRefPtr<CefFrame> frame,
                                        CefProcessId source_process,
                                        CefRefPtr<CefProcessMessage> message) override;

  // CefDisplayHandler methods:
  virtual void OnTitleChange(CefRefPtr<CefBrowser> browser,
                             const CefString &title) override;
  virtual void OnAddressChange(CefRefPtr<CefBrowser> browser,
                               CefRefPtr<CefFrame> frame,
                               const CefString &url) override;
  // virtual bool OnCursorChange(CefRefPtr<CefBrowser> browser,
  //                             CefCursorHandle cursor,
  //                             cef_cursor_type_t type,
  //                             const CefCursorInfo &custom_cursor_info) override;
  // CefLifeSpanHandler methods:
  virtual void OnAfterCreated(CefRefPtr<CefBrowser> browser) override;
  virtual bool DoClose(CefRefPtr<CefBrowser> browser) override;
  virtual void OnBeforeClose(CefRefPtr<CefBrowser> browser) override;
  virtual bool OnBeforePopup(CefRefPtr<CefBrowser> browser,
                             CefRefPtr<CefFrame> frame,
                             const CefString &target_url,
                             const CefString &target_frame_name,
                             WindowOpenDisposition target_disposition,
                             bool user_gesture,
                             const CefPopupFeatures &popupFeatures,
                             CefWindowInfo &windowInfo,
                             CefRefPtr<CefClient> &client,
                             CefBrowserSettings &settings,
                             CefRefPtr<CefDictionaryValue> &extra_info,
                             bool *no_javascript_access) override;

  // CefLoadHandler methods:
  virtual void OnLoadError(CefRefPtr<CefBrowser> browser,
                           CefRefPtr<CefFrame> frame,
                           ErrorCode errorCode,
                           const CefString &errorText,
                           const CefString &failedUrl) override;
  virtual void OnLoadStart(CefRefPtr<CefBrowser> browser,
                           CefRefPtr<CefFrame> frame,
                           TransitionType transition_type) override;
  virtual void OnLoadingStateChange(CefRefPtr<CefBrowser> browser,
                                    bool isLoading,
                                    bool canGoBack,
                                    bool canGoForward) override;
  virtual void OnLoadEnd(CefRefPtr<CefBrowser> browser,
                         CefRefPtr<CefFrame> frame,
                         int httpStatusCode) override;
  // CefRenderHandler  methods:
  virtual void GetViewRect(CefRefPtr<CefBrowser> browser, CefRect &rect) override;

  virtual void OnPaint(CefRefPtr<CefBrowser> browser,
                       PaintElementType type,
                       const RectList &dirtyRects,
                       const void *buffer,
                       int width,
                       int height) override;

  virtual bool GetScreenInfo(CefRefPtr<CefBrowser> browser,
                             CefScreenInfo &screen_info) override;

  virtual void OnTakeFocus(CefRefPtr<CefBrowser> browser, bool next) override;

  virtual bool OnSetFocus(CefRefPtr<CefBrowser> browser, FocusSource source) override;

  virtual void OnGotFocus(CefRefPtr<CefBrowser> browser) override;

  virtual void OnPopupShow(CefRefPtr<CefBrowser> browser, bool show) override;

  const FlutterDesktopPixelBuffer *CopyPixelBuffer(size_t width, size_t height)
  {
    return pixel_buffer_.get();
  }
  void SetTextureSize(int width, int height, double devicePixelRatio);
  void LoadUrl(const std::string &url);
  void LoadStringContent(const std::string &content);
  void Reload();
  void StopLoad();

  void GoBack();
  void GoForward();
  bool CanGoBack();
  bool CanGoForward();

  // webpage 位置信息
  void SetContentSize(int width, int height);
  void SetCursorOffset(int x, int y);
  void SetContentOffset(double x, double y);

  void Dispose();

  void SetWindowOpenPolicy(WebviewClientWindowOpenPolicy policy);

  //鼠标处理
  void SetMouseClick(WebviewClientPointerButton button, bool isDown, int x, int y);
  void SetMouseDoubleClick(WebviewClientPointerButton button, bool isDown, int x, int y);
  void SetMouseWheel(int x, int y, int deltaX, int deltaY);
  //输入事件的处理
  // void SetKeyEvent(WebviewClientKeyEventType eventType, int windowsKeyCode, int nativekeycode);
  void OnKeyEvent(UINT message, WPARAM const wparam,
                  LPARAM const lparam);
  //
  void SetCustomUserAgent(const std::string &user_agent);

  //执行js
  void EvaluateJavaScript(const std::string &url, const std::string &jsCode);
  void CallCppHanlder(const std::string &name, const std::string &result);

  void OnCursorChanged(CursorChangedCallback callback)
  {
    cursor_changed_callback_ = std::move(callback);
  }
  void OnLoadStateChanged(LoadStateChangedCallback callback)
  {
    load_state_callback_ = std::move(callback);
  }
  void OnHistoryChanged(HistoryChangedCallback callback)
  {
    history_changed_callback_ = std::move(callback);
  }
  void OnUrlChanged(UrlChangedCallback callback)
  {
    url_changed_callback_ = std::move(callback);
  }



  void OnJSHandleChanged(JSHandleCallback callback)
  {
    js_handle_changed_callback_ = std::move(callback);
  }

  // Returns true if the Chrome runtime is enabled.
  static bool IsChromeRuntimeEnabled();

private:
  // Platform-specific implementation.
  void PlatformTitleChange(CefRefPtr<CefBrowser> browser,
                           const CefString &title);
  CefRefPtr<CefBrowser> GetBrowser();

  // True if the application is using the Views framework.
  const bool use_views_;

  std::shared_ptr<uint8_t> sw_buffer_;
  std::shared_ptr<uint8_t> backing_pixel_buffer_;
  std::unique_ptr<FlutterDesktopPixelBuffer> pixel_buffer_ = {};
  Size texture_size_ = {0, 0, 1.0};
  FrameAvailableCallback frame_available_;
  WebviewClientWindowOpenPolicy popup_window_policy_;

  // List of existing browser windows. Only accessed on the CEF UI thread.
  // typedef std::list<CefRefPtr<CefBrowser>>
  //     BrowserList;
  // BrowserList browser_list_;

  CefRefPtr<CefBrowser> browser_;

  bool is_closing_;

  CursorChangedCallback cursor_changed_callback_;
  LoadStateChangedCallback load_state_callback_;
  HistoryChangedCallback history_changed_callback_;
  UrlChangedCallback url_changed_callback_;
  JSHandleCallback js_handle_changed_callback_;

  // Include the default reference counting implementation.
  IMPLEMENT_REFCOUNTING(WebviewClient);
};

#endif // CEF_TESTS_CEFSIMPLE_SIMPLE_HANDLER_H_
